/*
 * pclDepthRead.cpp
 * Retrieves the depth frame from the connected camera, and imports it into Point Cloud Library for displaying.
 * Author: Cooper Lawrence, Metavues Tech
 * Last updated: 29/05/2019
 */

#include <MarsSDK.h>

#include <pcl/common/common_headers.h>
#include <pcl/features/normal_3d.h>
#include <pcl/io/pcd_io.h>
#include <pcl/visualization/pcl_visualizer.h>

//Sets up the point cloud visualizer
boost::shared_ptr<pcl::visualization::PCLVisualizer> createViewer(pcl::PointCloud<pcl::PointXYZ>::ConstPtr cloud) {
	boost::shared_ptr<pcl::visualization::PCLVisualizer> viewer(new pcl::visualization::PCLVisualizer("3D Viewer")); //Initialize a new point cloud visualizer

	viewer->setBackgroundColor(0, 0, 0); //Set the background color to black
	viewer->addPointCloud<pcl::PointXYZ>(cloud, "cloud"); //Add the point cloud to the visualizer
	viewer->setPointCloudRenderingProperties(pcl::visualization::PCL_VISUALIZER_POINT_SIZE, 1, "cloud"); //Set the point size
	//viewer->addCoordinateSystem(1.0); //Add the coordinate axis at the origin
	viewer->initCameraParameters(); //Finalize camera parameters

	return (viewer);
}

int main()
{
	Mars* device = new Mars(); //Create a new Mars camera instance
	device->connect();

	DepthStream* depthStream = device->getDepthStream(); //Create the camera's depth stream

	pcl::PointCloud<pcl::PointXYZ>::Ptr cloud(new pcl::PointCloud<pcl::PointXYZ>); //Initialize the point cloud

	cloud->width = depthStream->getWidth();
	cloud->height = depthStream->getHeight();
	cloud->is_dense = false; //Specifies that the point cloud's data isn't finite (it's possible for points to contain infinit/NaN values)
	cloud->points.resize(depthStream->getSize());

	boost::shared_ptr<pcl::visualization::PCLVisualizer> viewer = createViewer(cloud); //Visualize the point cloud

	//Load the depth frame buffer from the camera
	while (!viewer->wasStopped())
	{
		depthStream->update(); //Update the depth stream

		//Update the point cloud's contents
		for (size_t y = 0; y < depthStream->getHeight(); y++) {
			for (size_t x = 0; x < depthStream->getWidth(); x++) {
				//Depth data (Coordinate system is mirrored by default, so negate X and Y values)
				cloud->points[(y * depthStream->getWidth()) + x].x = -depthStream->getFrame()[(y * depthStream->getWidth()) + x].x;
				cloud->points[(y * depthStream->getWidth()) + x].y = -depthStream->getFrame()[(y * depthStream->getWidth()) + x].y;
				cloud->points[(y * depthStream->getWidth()) + x].z = depthStream->getFrame()[(y * depthStream->getWidth()) + x].z;
			}
		}

		viewer->updatePointCloud(cloud, "cloud"); //Update the point cloud in the visualizer

		viewer->spinOnce(0); //Give the visualizer time to handle user input	
	}

	device->closeDepthStream();
	device->disconnect();

	return 0;
}
