// MarsSDK.cpp : Defines the exported functions for the library.
//

#include <MarsSDK.h>

#define ROUND_UP(x, align) (x+(align-1))&(~(align-1))

int Stream::getWidth()
{
	return width;
}

int Stream::getHeight()
{
	return height;
}

int Stream::getSize()
{
	return size;
}

DepthStream::DepthStream(CLibTof* _camera)
{
	camera = _camera;
	width = 224;
	height = 172;
	size = width * (height + 1);
	depthFrame = new FrameData_t[size];
	colorFrame = new FrameDataRgb_t[size];
	irFrame = new unsigned short[width * height];
	//pseudoColor = new unsigned char[size * 4];
}

DepthStream::~DepthStream()
{
	//Destructor
}

int DepthStream::update()
{
	unsigned char* tmp = new unsigned char[width * height * 2];

	int code = camera->LibTOF_RcvDepthFrame(depthFrame, colorFrame, tmp);
	if (code < 0)
	{
		fprintf(stderr, "LibTOF_RcvDepthFrame Failed: %d\n", code);
	}

	//Combine two unsigned chars into an unsigned short, and then normalize the 12 bit grayscale to 16 bit grayscale
	for (int i = 0; i < width * height; i++)
	{
		irFrame[i] = ((tmp[i * 2 + 1] << 8) | tmp[i * 2]) << 4;
	}

	return code;
}

/*
int DepthStream::updatePseudoColor(float _min, float _max)
{
	int code = camera->LibTOF_PseudoColor(depthFrame, width, height, pseudoColor, _min, _max);
        if (code < 0)
        {
                fprintf(stderr, "LibTOF_PseudoColor Failed: %d\n", code);
        }

	return code;
}
*/

FrameData_t* DepthStream::getFrame()
{ 
	return depthFrame;
}

FrameDataRgb_t* DepthStream::getColorFrame()
{
	return colorFrame;
}

unsigned short* DepthStream::getIrFrame()
{
	return irFrame;
}

/*
unsigned char* DepthStream::getPseudoColorFrame()
{
	return pseudoColor;
}
*/

ColorStream::ColorStream(CLibTof* _camera)
{
	camera = _camera;
	width = 1344;
	height = 760;
	size = ROUND_UP(width * height * 3 / 2 + width, 1024);
	frame = new short[size];
}

ColorStream::~ColorStream()
{
	//Destructor
}

int ColorStream::update()
{
	int code = camera->LibTOF_RcvRgbFrame(frame, size);
        if (code < 0)
        {
                fprintf(stderr, "LibTOF_RcvVideoFrame Failed: %d\n", code);
        }

	return code;
}

short* ColorStream::getFrame()
{
	return frame;
}

Mars::Mars()
{
	camera = CLibTof::LibTOF_NewInstance();
	deviceInfo = new DeviceInfo_t();

	depthStream = NULL;
	colorStream = NULL;
}

Mars::~Mars()
{
	//Destructor
}

int Mars::connect()
{
	//Connect the device
	int code = camera->LibTOF_Connect();
	if (code < 0)
	{
		fprintf(stderr, "LibTOF_Connect Failed: %d\n", code);
		return code;
	}

	//Retrieve the device information
	code = camera->LibTOF_GetDeviceInfo(deviceInfo);
	if (code < 0)
	{
		fprintf(stderr, "LibTOF_GetDeviceInfo Failed: %d\n", code);
		return code;
	}

	//Set the RGBD capture switch
	code = camera->LibTOF_RgbdEnSet(0);
	if (code < 0)
	{
		fprintf(stderr, "LibTOF_RgbdEnSet Failed: %d\n", code);
		return code;
	}

	//Set the depth camera mode
	code = camera->LibTOF_Setparam(SET_CAMERA_MODE, 2);
	if (code < 0)
	{
		fprintf(stderr, "LibTOF_Setparam Failed: %d\n", code);
		return code;
	}

	//Set the camera to not use a preset scene mode
	code = camera->LibTOF_SetSceneMode(Scene_No);
	if (code < 0)
	{
		fprintf(stderr, "LibTOF_SetSceneMode Failed: %d\n", code);
		return code;
	}

	//Set the camera framerate to 30fps
	code = camera->LibTOF_Setparam(SET_CAMERA_FPS, 30);
	if (code < 0)
	{
		fprintf(stderr, "LibTOF_Setparam Failed: %d\n", code);
	}

	//Set the camera exposure to be 
	code = camera->LibTOF_SetExpourseMode(Manual);
	if (code < 0)
	{
		fprintf(stderr, "LibTOF_SetExposureMode Failed: %d\n", code);
	}

	code = camera->LibTOF_Setparam(SET_CAMERA_EXP, 380);
	if (code < 0)
	{
		fprintf(stderr, "LibTOF_Setparam Failed: %d\n", code);
	}

	return 0;
}

int Mars::disconnect()
{
	return camera->LibTOF_DisConnect();
}

DepthStream* Mars::getDepthStream()
{
	if (depthStream == NULL)
	{
		depthStream = new DepthStream(camera);
	}

	return depthStream;
}

ColorStream* Mars::getColorStream()
{
	if (colorStream == NULL)
	{
		colorStream = new ColorStream(camera);
	}

	return colorStream;
}

int Mars::closeDepthStream()
{
	if (depthStream != NULL)
	{
		delete depthStream;
		return 0;
	}

	return -1;
}

int Mars::closeColorStream()
{
	if (colorStream != NULL)
	{
		delete colorStream;
		return 0;
	}

	return -1;
}

unsigned int Mars::getDeviceVersion()
{
	return deviceInfo->DeviceVersion;
}

unsigned int Mars::getDeviceType()
{
	return deviceInfo->DeviceType;
}

char* Mars::getDeviceId()
{
	return deviceInfo->DeviceId;
}

char* Mars::getDriverVersion()
{
	return deviceInfo->TofAlgVersion;
}

int Mars::getTemp(float *_cpu, float *_sensor)
{
	int code = camera->LibTOF_GetTemperture(_cpu, _sensor);
	if (code < 0)
	{
		fprintf(stderr, "LibTOF_GetTemperature Failed: %d\n", code);
	}

	return code;
}

int Mars::setCameraMode(int _mode)
{
	//0 = mono - frequency, 1 = multi - frequency, 2 = IR data
	int code = camera->LibTOF_Setparam(SET_CAMERA_MODE, _mode);
	if (code < 0)
	{
		fprintf(stderr, "LibTOF_Setparam Failed: %d\n", code);
	}

	return code;
}

int Mars::setFps(int _fps)
{
	//5, 15, 30, or 45
	int code = camera->LibTOF_Setparam(SET_CAMERA_FPS, _fps);
	if (code < 0)
	{
		fprintf(stderr, "LibTOF_Setparam Failed: %d\n", code);
	}

	return code;
}

int Mars::setExposure(int _exposure)
{
	int code;

	if (_exposure > 0)
	{
		code = camera->LibTOF_SetExpourseMode(Manual);
		if (code < 0)
		{
			fprintf(stderr, "LibTOF_SetExposureMode Failed: %d\n", code);
		}

		code = camera->LibTOF_Setparam(SET_CAMERA_EXP, _exposure);
		if (code < 0)
		{
			fprintf(stderr, "LibTOF_Setparam Failed: %d\n", code);
		}
	}
	else
	{
		code = camera->LibTOF_SetExpourseMode(Auto);
		if (code < 0)
		{
			fprintf(stderr, "LibTOF_SetExposureMode Failed: %d\n", code);
		}
	}

	return code;
}

/*
int Mars::setIrExposure(int _exposure)
{
	int code;

	code = camera->LibTOF_Setparam(SET_GREY_EXP, _exposure);
	if (code < 0)
	{
		fprintf(stderr, "LibTOF_Setparam Failed: %d\n", code);
	}

	return code;
}
*/

int Mars::setImageRegistration(bool _status)
{
	int code = camera->LibTOF_RgbdEnSet(_status);
	if (code < 0)
	{
		fprintf(stderr, "LibTOF_RgbdEnSet Failed: %d\n", code);
	}

	return code;
}
